@@ -20,10 +20,8 @@ Command-line interface to the OpenStack Nova API.
 
 from __future__ import print_function
 import argparse
-import getpass
 import logging
 import sys
-import warnings
 
 from keystoneauth1 import loading
 from oslo_utils import encodeutils
@@ -31,22 +29,14 @@ from oslo_utils import importutils
 from oslo_utils import strutils
 import six
 
-HAS_KEYRING = False
-all_errors = ValueError
-try:
-    import keyring
-    HAS_KEYRING = True
-except ImportError:
-    pass
+osprofiler_profiler = importutils.try_import("osprofiler.profiler")
 
 import novaclient
 from novaclient import api_versions
-import novaclient.auth_plugin
 from novaclient import client
 from novaclient import exceptions as exc
 import novaclient.extension
 from novaclient.i18n import _
-from novaclient.openstack.common import cliutils
 from novaclient import utils
 
 DEFAULT_MAJOR_OS_COMPUTE_API_VERSION = "2.0"
@@ -211,148 +201,6 @@ class DeprecatedAction(argparse.Action):
             action(parser, namespace, values, option_string)
 
 
-def positive_non_zero_float(text):
-    if text is None:
-        return None
-    try:
-        value = float(text)
-    except ValueError:
-        msg = _("%s must be a float") % text
-        raise argparse.ArgumentTypeError(msg)
-    if value <= 0:
-        msg = _("%s must be greater than 0") % text
-        raise argparse.ArgumentTypeError(msg)
-    return value
-
-
-class SecretsHelper(object):
-    def __init__(self, args, client):
-        self.args = args
-        self.client = client
-        self.key = None
-        self._password = None
-
-    def _validate_string(self, text):
-        if text is None or len(text) == 0:
-            return False
-        return True
-
-    def _make_key(self):
-        if self.key is not None:
-            return self.key
-        keys = [
-            self.client.auth_url,
-            self.client.projectid,
-            self.client.user,
-            self.client.region_name,
-            self.client.endpoint_type,
-            self.client.service_type,
-            self.client.service_name,
-            self.client.volume_service_name,
-        ]
-        for (index, key) in enumerate(keys):
-            if key is None:
-                keys[index] = '?'
-            else:
-                keys[index] = str(keys[index])
-        self.key = "/".join(keys)
-        return self.key
-
-    def _prompt_password(self, verify=True):
-        pw = None
-        if hasattr(sys.stdin, 'isatty') and sys.stdin.isatty():
-            # Check for Ctl-D
-            try:
-                while True:
-                    pw1 = getpass.getpass('OS Password: ')
-                    if verify:
-                        pw2 = getpass.getpass('Please verify: ')
-                    else:
-                        pw2 = pw1
-                    if pw1 == pw2 and self._validate_string(pw1):
-                        pw = pw1
-                        break
-            except EOFError:
-                pass
-        return pw
-
-    def save(self, auth_token, management_url, tenant_id):
-        if not HAS_KEYRING or not self.args.os_cache:
-            return
-        if (auth_token == self.auth_token and
-                management_url == self.management_url):
-            # Nothing changed....
-            return
-        if not all([management_url, auth_token, tenant_id]):
-            raise ValueError(_("Unable to save empty management url/auth "
-                               "token"))
-        value = "|".join([str(auth_token),
-                          str(management_url),
-                          str(tenant_id)])
-        keyring.set_password("novaclient_auth", self._make_key(), value)
-
-    @property
-    def password(self):
-        # Cache password so we prompt user at most once
-        if self._password:
-            pass
-        elif self._validate_string(self.args.os_password):
-            self._password = self.args.os_password
-        else:
-            verify_pass = strutils.bool_from_string(
-                utils.env("OS_VERIFY_PASSWORD", default=False), True)
-            self._password = self._prompt_password(verify_pass)
-        if not self._password:
-            raise exc.CommandError(
-                'Expecting a password provided via either '
-                '--os-password, env[OS_PASSWORD], or '
-                'prompted response')
-        return self._password
-
-    @property
-    def management_url(self):
-        if not HAS_KEYRING or not self.args.os_cache:
-            return None
-        management_url = None
-        try:
-            block = keyring.get_password('novaclient_auth', self._make_key())
-            if block:
-                _token, management_url, _tenant_id = block.split('|', 2)
-        except all_errors:
-            pass
-        return management_url
-
-    @property
-    def auth_token(self):
-        # Now is where it gets complicated since we
-        # want to look into the keyring module, if it
-        # exists and see if anything was provided in that
-        # file that we can use.
-        if not HAS_KEYRING or not self.args.os_cache:
-            return None
-        token = None
-        try:
-            block = keyring.get_password('novaclient_auth', self._make_key())
-            if block:
-                token, _management_url, _tenant_id = block.split('|', 2)
-        except all_errors:
-            pass
-        return token
-
-    @property
-    def tenant_id(self):
-        if not HAS_KEYRING or not self.args.os_cache:
-            return None
-        tenant_id = None
-        try:
-            block = keyring.get_password('novaclient_auth', self._make_key())
-            if block:
-                _token, _management_url, tenant_id = block.split('|', 2)
-        except all_errors:
-            pass
-        return tenant_id
-
-
 class NovaClientArgumentParser(argparse.ArgumentParser):
 
     def __init__(self, *args, **kwargs):
@@ -395,20 +243,23 @@ class NovaClientArgumentParser(argparse.ArgumentParser):
 class OpenStackComputeShell(object):
     times = []
 
+    def __init__(self):
+        self.client_logger = None
+
     def _append_global_identity_args(self, parser, argv):
         # Register the CLI arguments that have moved to the session object.
         loading.register_session_argparse_arguments(parser)
-        # Peek into argv to see if os-auth-token or os-token were given,
+        # Peek into argv to see if os-token was given,
         # in which case, the token auth plugin is what the user wants
         # else, we'll default to password
         default_auth_plugin = 'password'
-        if 'os-token' in argv:
+        if "--os-token" in argv:
             default_auth_plugin = 'token'
         loading.register_auth_argparse_arguments(
             parser, argv, default=default_auth_plugin)
 
-        parser.set_defaults(insecure=utils.env('NOVACLIENT_INSECURE',
-                            default=False))
+        parser.set_defaults(insecure=strutils.bool_from_string(
+            utils.env('NOVACLIENT_INSECURE', default=False)))
         parser.set_defaults(os_auth_url=utils.env('OS_AUTH_URL', 'NOVA_URL'))
 
         parser.set_defaults(os_username=utils.env('OS_USERNAME',
@@ -419,6 +270,14 @@ class OpenStackComputeShell(object):
             'OS_PROJECT_NAME', 'OS_TENANT_NAME', 'NOVA_PROJECT_ID'))
         parser.set_defaults(os_project_id=utils.env(
             'OS_PROJECT_ID', 'OS_TENANT_ID'))
+        parser.set_defaults(
+            os_project_domain_id=utils.env('OS_PROJECT_DOMAIN_ID'))
+        parser.set_defaults(
+            os_project_domain_name=utils.env('OS_PROJECT_DOMAIN_NAME'))
+        parser.set_defaults(
+            os_user_domain_id=utils.env('OS_USER_DOMAIN_ID'))
+        parser.set_defaults(
+            os_user_domain_name=utils.env('OS_USER_DOMAIN_NAME'))
 
     def get_base_parser(self, argv):
         parser = NovaClientArgumentParser(
@@ -461,92 +320,22 @@ class OpenStackComputeShell(object):
             action='store_true',
             help=_("Print call timing info."))
 
-        parser.add_argument(
-            '--os_username',
-            action=DeprecatedAction,
-            use=_('use "%s"; this option will be removed '
-                  'in novaclient 3.3.0.') % '--os-username',
-            help=argparse.SUPPRESS)
-
-        parser.add_argument(
-            '--os_password',
-            action=DeprecatedAction,
-            use=_('use "%s"; this option will be removed '
-                  'in novaclient 3.3.0.') % '--os-password',
-            help=argparse.SUPPRESS)
-
-        parser.add_argument(
-            '--os_tenant_name',
-            action=DeprecatedAction,
-            use=_('use "%s"; this option will be removed '
-                  'in novaclient 3.3.0.') % '--os-tenant-name',
-            help=argparse.SUPPRESS)
-
-        parser.add_argument(
-            '--os_auth_url',
-            action=DeprecatedAction,
-            use=_('use "%s"; this option will be removed '
-                  'in novaclient 3.3.0.') % '--os-auth-url',
-            help=argparse.SUPPRESS)
-
         parser.add_argument(
             '--os-region-name',
             metavar='<region-name>',
             default=utils.env('OS_REGION_NAME', 'NOVA_REGION_NAME'),
             help=_('Defaults to env[OS_REGION_NAME].'))
-        parser.add_argument(
-            '--os_region_name',
-            action=DeprecatedAction,
-            use=_('use "%s"; this option will be removed '
-                  'in novaclient 3.3.0.') % '--os-region-name',
-            help=argparse.SUPPRESS)
-
-        parser.add_argument(
-            '--os-auth-system',
-            metavar='<auth-system>',
-            default=utils.env('OS_AUTH_SYSTEM'),
-            help=argparse.SUPPRESS)
-        parser.add_argument(
-            '--os_auth_system',
-            action=DeprecatedAction,
-            use=_('use "%s"; this option will be removed '
-                  'in novaclient 3.3.0.') % '--os-auth-system',
-            help=argparse.SUPPRESS)
 
         parser.add_argument(
             '--service-type',
             metavar='<service-type>',
             help=_('Defaults to compute for most actions.'))
-        parser.add_argument(
-            '--service_type',
-            action=DeprecatedAction,
-            use=_('use "%s"; this option will be removed '
-                  'in novaclient 3.3.0.') % '--service-type',
-            help=argparse.SUPPRESS)
 
         parser.add_argument(
             '--service-name',
             metavar='<service-name>',
             default=utils.env('NOVA_SERVICE_NAME'),
             help=_('Defaults to env[NOVA_SERVICE_NAME].'))
-        parser.add_argument(
-            '--service_name',
-            action=DeprecatedAction,
-            use=_('use "%s"; this option will be removed '
-                  'in novaclient 3.3.0.') % '--service-name',
-            help=argparse.SUPPRESS)
-
-        parser.add_argument(
-            '--volume-service-name',
-            metavar='<volume-service-name>',
-            default=utils.env('NOVA_VOLUME_SERVICE_NAME'),
-            help=_('Defaults to env[NOVA_VOLUME_SERVICE_NAME].'))
-        parser.add_argument(
-            '--volume_service_name',
-            action=DeprecatedAction,
-            use=_('use "%s"; this option will be removed '
-                  'in novaclient 3.3.0.') % '--volume-service-name',
-            help=argparse.SUPPRESS)
 
         parser.add_argument(
             '--os-endpoint-type',
@@ -560,19 +349,6 @@ class OpenStackComputeShell(object):
                    'env[OS_ENDPOINT_TYPE] or ') +
                  DEFAULT_NOVA_ENDPOINT_TYPE + '.')
 
-        parser.add_argument(
-            '--endpoint-type',
-            action=DeprecatedAction,
-            use=_('use "%s"; this option will be removed '
-                  'in novaclient 3.3.0.') % '--os-endpoint-type',
-            help=argparse.SUPPRESS)
-        # NOTE(dtroyer): We can't add --endpoint_type here due to argparse
-        #                thinking usage-list --end is ambiguous; but it
-        #                works fine with only --endpoint-type present
-        #                Go figure.  I'm leaving this here for doc purposes.
-        # parser.add_argument('--endpoint_type',
-        #     help=argparse.SUPPRESS)
-
         parser.add_argument(
             '--os-compute-api-version',
             metavar='<compute-api-ver>',
@@ -580,29 +356,50 @@ class OpenStackComputeShell(object):
                               default=DEFAULT_OS_COMPUTE_API_VERSION),
             help=_('Accepts X, X.Y (where X is major and Y is minor part) or '
                    '"X.latest", defaults to env[OS_COMPUTE_API_VERSION].'))
-        parser.add_argument(
-            '--os_compute_api_version',
-            action=DeprecatedAction,
-            use=_('use "%s"; this option will be removed '
-                  'in novaclient 3.3.0.') % '--os-compute-api-version',
-            help=argparse.SUPPRESS)
 
         parser.add_argument(
-            '--bypass-url',
+            '--os-endpoint-override',
             metavar='<bypass-url>',
-            dest='bypass_url',
-            default=utils.env('NOVACLIENT_BYPASS_URL'),
+            dest='endpoint_override',
+            default=utils.env('OS_ENDPOINT_OVERRIDE',
+                              'NOVACLIENT_ENDPOINT_OVERRIDE',
+                              'NOVACLIENT_BYPASS_URL'),
             help=_("Use this API endpoint instead of the Service Catalog. "
-                   "Defaults to env[NOVACLIENT_BYPASS_URL]."))
+                   "Defaults to env[OS_ENDPOINT_OVERRIDE]."))
+
+        # NOTE(takashin): This dummy '--end' argument was added
+        # to avoid misinterpreting command line arguments.
+        # If there is not this dummy argument, the '--end' is interpreted to
+        # the '--endpoint-override'.
+        # TODO(takashin): Remove this dummy '--end' argument
+        # when the deprecated '--endpoint-override' argument is removed.
         parser.add_argument(
-            '--bypass_url',
-            action=DeprecatedAction,
-            use=_('use "%s"; this option will be removed '
-                  'in novaclient 3.3.0.') % '--bypass-url',
+            '--end',
+            metavar='<end>',
+            nargs='?',
             help=argparse.SUPPRESS)
 
-        # The auth-system-plugins might require some extra options
-        novaclient.auth_plugin.load_auth_system_opts(parser)
+        parser.add_argument(
+            '--endpoint-override',
+            action=DeprecatedAction,
+            use=_('use "%s"; this option will be removed after Rocky '
+                  'OpenStack release.') % '--os-endpoint-override',
+            dest='endpoint_override',
+            help=argparse.SUPPRESS)
+
+        if osprofiler_profiler:
+            parser.add_argument('--profile',
+                                metavar='HMAC_KEY',
+                                default=utils.env('OS_PROFILE'),
+                                help='HMAC key to use for encrypting context '
+                                'data for performance profiling of operation. '
+                                'This key should be the value of the HMAC key '
+                                'configured for the OSprofiler middleware in '
+                                'nova; it is specified in the Nova '
+                                'configuration file at "/etc/nova/nova.conf". '
+                                'Without the key, profiling will not be '
+                                'triggered even if OSprofiler is enabled on '
+                                'the server side.')
 
         self._append_global_identity_args(parser, argv)
 
@@ -645,8 +442,7 @@ class OpenStackComputeShell(object):
             desc = callback.__doc__ or ''
             if hasattr(callback, "versioned"):
                 additional_msg = ""
-                subs = api_versions.get_substitutions(
-                    utils.get_function_name(callback))
+                subs = api_versions.get_substitutions(callback)
                 if do_help:
                     additional_msg = msg % {
                         'start': subs[0].start_version.get_string(),
@@ -713,44 +509,24 @@ class OpenStackComputeShell(object):
                             format=streamformat)
         logging.getLogger('iso8601').setLevel(logging.WARNING)
 
+        self.client_logger = logging.getLogger(client.__name__)
+        ch = logging.StreamHandler()
+        self.client_logger.setLevel(logging.DEBUG)
+        self.client_logger.addHandler(ch)
+
     def main(self, argv):
         # Parse args once to find version and debug settings
         parser = self.get_base_parser(argv)
-
-        # NOTE(dtroyer): Hackery to handle --endpoint_type due to argparse
-        #                thinking usage-list --end is ambiguous; but it
-        #                works fine with only --endpoint-type present
-        #                Go figure.
-        if '--endpoint_type' in argv:
-            spot = argv.index('--endpoint_type')
-            argv[spot] = '--endpoint-type'
-            # NOTE(Vek): Not emitting a warning here, as that will
-            #            occur when "--endpoint-type" is processed
-
-        # For backwards compat with old os-auth-token parameter
-        if '--os-auth-token' in argv:
-            spot = argv.index('--os-auth-token')
-            argv[spot] = '--os-token'
-            print(_('WARNING: Option "%(option)s" is deprecated; %(use)s') % {
-                'option': '--os-auth-token',
-                'use': _('use "%s"; this option will be removed in '
-                         'novaclient 3.3.0.') % '--os-token',
-            }, file=sys.stderr)
-
         (args, args_list) = parser.parse_known_args(argv)
 
         self.setup_debugging(args.debug)
         self.extensions = []
-        do_help = ('help' in argv) or (
-            '--help' in argv) or ('-h' in argv) or not argv
+        do_help = args.help or not args_list or args_list[0] == 'help'
 
-        # bash-completion should not require authentification
+        # bash-completion should not require authentication
         skip_auth = do_help or (
             'bash-completion' in argv)
 
-        # Discover available auth plugins
-        novaclient.auth_plugin.discover_auth_systems()
-
         if not args.os_compute_api_version:
             api_version = api_versions.get_api_version(
                 DEFAULT_MAJOR_OS_COMPUTE_API_VERSION)
@@ -758,8 +534,10 @@ class OpenStackComputeShell(object):
             api_version = api_versions.get_api_version(
                 args.os_compute_api_version)
 
-        os_username = args.os_username
-        os_user_id = args.os_user_id
+        auth_token = getattr(args, "os_token", None)
+
+        os_username = getattr(args, "os_username", None)
+        os_user_id = getattr(args, "os_user_id", None)
         os_password = None  # Fetched and set later as needed
         os_project_name = getattr(
             args, 'os_project_name', getattr(args, 'os_tenant_name', None))
@@ -767,38 +545,37 @@ class OpenStackComputeShell(object):
             args, 'os_project_id', getattr(args, 'os_tenant_id', None))
         os_auth_url = args.os_auth_url
         os_region_name = args.os_region_name
-        os_auth_system = args.os_auth_system
+
+        if "v2.0" not in os_auth_url:
+            # NOTE(andreykurilin): assume that keystone V3 is used and try to
+            # be more user-friendly, i.e provide default values for domains
+            if (not args.os_project_domain_id and
+                    not args.os_project_domain_name):
+                setattr(args, "os_project_domain_id", "default")
+
+            # os_user_domain_id is redundant in case of Token auth type
+            if not auth_token and (not args.os_user_domain_id and
+                                   not args.os_user_domain_name):
+                setattr(args, "os_user_domain_id", "default")
+
+        os_project_domain_id = args.os_project_domain_id
+        os_project_domain_name = args.os_project_domain_name
+        os_user_domain_id = getattr(args, "os_user_domain_id", None)
+        os_user_domain_name = getattr(args, "os_user_domain_name", None)
+
         endpoint_type = args.endpoint_type
         insecure = args.insecure
         service_type = args.service_type
         service_name = args.service_name
-        volume_service_name = args.volume_service_name
-        bypass_url = args.bypass_url
+        endpoint_override = args.endpoint_override
         os_cache = args.os_cache
         cacert = args.os_cacert
+        cert = args.os_cert
         timeout = args.timeout
 
         keystone_session = None
         keystone_auth = None
 
-        # We may have either, both or none of these.
-        # If we have both, we don't need USERNAME, PASSWORD etc.
-        # Fill in the blanks from the SecretsHelper if possible.
-        # Finally, authenticate unless we have both.
-        # Note if we don't auth we probably don't have a tenant ID so we can't
-        # cache the token.
-        auth_token = getattr(args, 'os_token', None)
-        management_url = bypass_url if bypass_url else None
-
-        if os_auth_system and os_auth_system != "keystone":
-            warnings.warn(_(
-                'novaclient auth plugins that are not keystone are deprecated.'
-                ' Auth plugins should now be done as plugins to keystoneauth'
-                ' and selected with --os-auth-type or OS_AUTH_TYPE'))
-            auth_plugin = novaclient.auth_plugin.load_plugin(os_auth_system)
-        else:
-            auth_plugin = None
-
         if not endpoint_type:
             endpoint_type = DEFAULT_NOVA_ENDPOINT_TYPE
 
@@ -812,29 +589,20 @@ class OpenStackComputeShell(object):
             # service type specified, we use default nova service type.
             service_type = DEFAULT_NOVA_SERVICE_TYPE
 
-        # If we have an auth token but no management_url, we must auth anyway.
+        # We should always auth unless we have a token and we're passing a
+        # specific endpoint
         # Expired tokens are handled by client.py:_cs_request
-        must_auth = not (auth_token and management_url)
-
-        # Do not use Keystone session for cases with no session support. The
-        # presence of auth_plugin means os_auth_system is present and is not
-        # keystone.
-        use_session = True
-        if auth_plugin or bypass_url or os_cache or volume_service_name:
-            use_session = False
+        must_auth = not (auth_token and endpoint_override)
 
         # FIXME(usrleon): Here should be restrict for project id same as
         # for os_username or os_password but for compatibility it is not.
         if must_auth and not skip_auth:
-            if auth_plugin:
-                auth_plugin.parse_opts(args)
 
-            if not auth_plugin or not auth_plugin.opts:
-                if not os_username and not os_user_id:
-                    raise exc.CommandError(
-                        _("You must provide a username "
-                          "or user ID via --os-username, --os-user-id, "
-                          "env[OS_USERNAME] or env[OS_USER_ID]"))
+            if not any([auth_token, os_username, os_user_id]):
+                raise exc.CommandError(
+                    _("You must provide a user name/id (via --os-username, "
+                      "--os-user-id, env[OS_USERNAME] or env[OS_USER_ID]) or "
+                      "an auth token (via --os-token)."))
 
             if not any([os_project_name, os_project_id]):
                 raise exc.CommandError(_("You must provide a project name or"
@@ -845,28 +613,36 @@ class OpenStackComputeShell(object):
                                          " interchangeably."))
 
             if not os_auth_url:
-                if os_auth_system and os_auth_system != 'keystone':
-                    os_auth_url = auth_plugin.get_auth_url()
+                raise exc.CommandError(
+                    _("You must provide an auth url "
+                      "via either --os-auth-url or env[OS_AUTH_URL]."))
 
-            if not os_auth_url:
-                    raise exc.CommandError(
-                        _("You must provide an auth url "
-                          "via either --os-auth-url or env[OS_AUTH_URL] "
-                          "or specify an auth_system which defines a "
-                          "default url with --os-auth-system "
-                          "or env[OS_AUTH_SYSTEM]"))
-
-            if use_session:
-                # Not using Nova auth plugin, so use keystone
-                with utils.record_time(self.times, args.timings,
-                                       'auth_url', args.os_auth_url):
-                    keystone_session = (
-                        loading.load_session_from_argparse_arguments(args))
-                    keystone_auth = (
-                        loading.load_auth_from_argparse_arguments(args))
-            else:
-                # set password for auth plugins
-                os_password = args.os_password
+            # TODO(Shilpasd): need to provide support in python - novaclient
+            # for required options for below default auth type plugins:
+            # 1. v3oidcclientcredential
+            # 2. v3oidcpassword
+            # 3. v3oidcauthcode
+            # 4. v3oidcaccesstoken
+            # 5. v3oauth1
+            # 6. v3fedkerb
+            # 7. v3adfspassword
+            # 8. v3samlpassword
+            # 9. v3applicationcredential
+            # TODO(Shilpasd): need to provide support in python - novaclient
+            # for below extra keystoneauth auth type plugins:
+            # We will need to add code to support discovering of versions
+            # supported by the keystone service based on the auth_url similar
+            # to the one supported by glanceclient.
+            # 1. v3password
+            # 2. v3token
+            # 3. v3kerberos
+            # 4. v3totp
+            with utils.record_time(self.times, args.timings,
+                                   'auth_url', args.os_auth_url):
+                keystone_session = (
+                    loading.load_session_from_argparse_arguments(args))
+                keystone_auth = (
+                    loading.load_auth_from_argparse_arguments(args))
 
         if (not skip_auth and
                 not any([os_project_name, os_project_id])):
@@ -882,22 +658,30 @@ class OpenStackComputeShell(object):
                 _("You must provide an auth url "
                   "via either --os-auth-url or env[OS_AUTH_URL]"))
 
+        additional_kwargs = {}
+        if osprofiler_profiler:
+            additional_kwargs["profile"] = args.profile
+
         # This client is just used to discover api version. Version API needn't
         # microversion, so we just pass version 2 at here.
         self.cs = client.Client(
             api_versions.APIVersion("2.0"),
-            os_username, os_password, os_project_name,
-            tenant_id=os_project_id, user_id=os_user_id,
+            os_username, os_password, project_id=os_project_id,
+            project_name=os_project_name, user_id=os_user_id,
             auth_url=os_auth_url, insecure=insecure,
             region_name=os_region_name, endpoint_type=endpoint_type,
             extensions=self.extensions, service_type=service_type,
-            service_name=service_name, auth_system=os_auth_system,
-            auth_plugin=auth_plugin, auth_token=auth_token,
-            volume_service_name=volume_service_name,
-            timings=args.timings, bypass_url=bypass_url,
+            service_name=service_name, auth_token=auth_token,
+            timings=args.timings, endpoint_override=endpoint_override,
             os_cache=os_cache, http_log_debug=args.debug,
-            cacert=cacert, timeout=timeout,
-            session=keystone_session, auth=keystone_auth)
+            cacert=cacert, cert=cert, timeout=timeout,
+            session=keystone_session, auth=keystone_auth,
+            logger=self.client_logger,
+            project_domain_id=os_project_domain_id,
+            project_domain_name=os_project_domain_name,
+            user_domain_id=os_user_domain_id,
+            user_domain_name=os_user_domain_name,
+            **additional_kwargs)
 
         if not skip_auth:
             if not api_version.is_latest():
@@ -943,69 +727,35 @@ class OpenStackComputeShell(object):
         if utils.isunauthenticated(args.func):
             # NOTE(alex_xu): We need authentication for discover microversion.
             # But the subcommands may needn't it. If the subcommand needn't,
-            # we clear the session arguements.
+            # we clear the session arguments.
             keystone_session = None
             keystone_auth = None
 
         # Recreate client object with discovered version.
         self.cs = client.Client(
             api_version,
-            os_username, os_password, os_project_name,
-            tenant_id=os_project_id, user_id=os_user_id,
+            os_username, os_password, project_id=os_project_id,
+            project_name=os_project_name, user_id=os_user_id,
             auth_url=os_auth_url, insecure=insecure,
             region_name=os_region_name, endpoint_type=endpoint_type,
             extensions=self.extensions, service_type=service_type,
-            service_name=service_name, auth_system=os_auth_system,
-            auth_plugin=auth_plugin, auth_token=auth_token,
-            volume_service_name=volume_service_name,
-            timings=args.timings, bypass_url=bypass_url,
+            service_name=service_name, auth_token=auth_token,
+            timings=args.timings, endpoint_override=endpoint_override,
             os_cache=os_cache, http_log_debug=args.debug,
-            cacert=cacert, timeout=timeout,
-            session=keystone_session, auth=keystone_auth)
-
-        # Now check for the password/token of which pieces of the
-        # identifying keyring key can come from the underlying client
-        if must_auth:
-            helper = SecretsHelper(args, self.cs.client)
-            self.cs.client.keyring_saver = helper
-            if (auth_plugin and auth_plugin.opts and
-                    "os_password" not in auth_plugin.opts):
-                use_pw = False
-            else:
-                use_pw = True
-
-            tenant_id = helper.tenant_id
-            # Allow commandline to override cache
-            if not auth_token:
-                auth_token = helper.auth_token
-            if not management_url:
-                management_url = helper.management_url
-            if tenant_id and auth_token and management_url:
-                self.cs.client.tenant_id = tenant_id
-                self.cs.client.auth_token = auth_token
-                self.cs.client.management_url = management_url
-                self.cs.client.password_func = lambda: helper.password
-            elif use_pw:
-                # We're missing something, so auth with user/pass and save
-                # the result in our helper.
-                self.cs.client.password = helper.password
-
-        try:
-            # This does a couple of bits which are useful even if we've
-            # got the token + service URL already. It exits fast in that case.
-            if not utils.isunauthenticated(args.func):
-                if not use_session:
-                    # Only call authenticate() if Nova auth plugin is used.
-                    # If keystone is used, authentication is handled as part
-                    # of session.
-                    self.cs.authenticate()
-        except exc.Unauthorized:
-            raise exc.CommandError(_("Invalid OpenStack Nova credentials."))
-        except exc.AuthorizationFailure:
-            raise exc.CommandError(_("Unable to authorize user"))
+            cacert=cacert, cert=cert, timeout=timeout,
+            session=keystone_session, auth=keystone_auth,
+            project_domain_id=os_project_domain_id,
+            project_domain_name=os_project_domain_name,
+            user_domain_id=os_user_domain_id,
+            user_domain_name=os_user_domain_name)
 
         args.func(self.cs, args)
 
+        if osprofiler_profiler and args.profile:
+            trace_id = osprofiler_profiler.get().get_base_id()
+            print("To display trace use the command:\n\n"
+                  "  osprofiler trace show --html %s " % trace_id)
+
         if args.timings:
             self._dump_timings(self.times + self.cs.get_timings())
 
@@ -1042,7 +792,7 @@ class OpenStackComputeShell(object):
         commands.remove('bash_completion')
         print(' '.join(commands | options))
 
-    @cliutils.arg(
+    @utils.arg(
         'command',
         metavar='<subcommand>',
         nargs='?',
@@ -1080,9 +830,13 @@ def main():
         OpenStackComputeShell().main(argv)
     except Exception as exc:
         logger.debug(exc, exc_info=1)
-        print(_("ERROR (%(type)s): %(msg)s") % {
+        if six.PY2:
+            message = encodeutils.safe_encode(six.text_type(exc))
+        else:
+            message = encodeutils.exception_to_unicode(exc)
+        print("ERROR (%(type)s): %(msg)s" % {
               'type': exc.__class__.__name__,
-              'msg': encodeutils.exception_to_unicode(exc)},
+              'msg': message},
               file=sys.stderr)
         sys.exit(1)
     except KeyboardInterrupt:
