@@ -21,9 +21,9 @@ import uuid
 
 from oslo_serialization import jsonutils
 from oslo_utils import encodeutils
-import pkg_resources
 import prettytable
 import six
+from six.moves.urllib import parse
 
 from novaclient import exceptions
 from novaclient.i18n import _
@@ -103,8 +103,11 @@ def add_arg(func, *args, **kwargs):
 
 def service_type(stype):
     """Adds 'service_type' attribute to decorated function.
+
     Usage:
+
     .. code-block:: python
+
        @service_type('volume')
        def mymethod(f):
        ...
@@ -115,43 +118,6 @@ def service_type(stype):
     return inner
 
 
-def add_resource_manager_extra_kwargs_hook(f, hook):
-    """Add hook to bind CLI arguments to ResourceManager calls.
-
-    The `do_foo` calls in shell.py will receive CLI args and then in turn pass
-    them through to the ResourceManager. Before passing through the args, the
-    hooks registered here will be called, giving us a chance to add extra
-    kwargs (taken from the command-line) to what's passed to the
-    ResourceManager.
-    """
-    if not hasattr(f, 'resource_manager_kwargs_hooks'):
-        f.resource_manager_kwargs_hooks = []
-
-    names = [h.__name__ for h in f.resource_manager_kwargs_hooks]
-    if hook.__name__ not in names:
-        f.resource_manager_kwargs_hooks.append(hook)
-
-
-def get_resource_manager_extra_kwargs(f, args, allow_conflicts=False):
-    """Return extra_kwargs by calling resource manager kwargs hooks."""
-    hooks = getattr(f, "resource_manager_kwargs_hooks", [])
-    extra_kwargs = {}
-    for hook in hooks:
-        hook_kwargs = hook(args)
-        hook_name = hook.__name__
-        conflicting_keys = set(hook_kwargs.keys()) & set(extra_kwargs.keys())
-        if conflicting_keys and not allow_conflicts:
-            msg = (_("Hook '%(hook_name)s' is attempting to redefine "
-                     "attributes '%(conflicting_keys)s'") %
-                   {'hook_name': hook_name,
-                    'conflicting_keys': conflicting_keys})
-            raise exceptions.NoUniqueMatch(msg)
-
-        extra_kwargs.update(hook_kwargs)
-
-    return extra_kwargs
-
-
 def pretty_choice_list(l):
     return ', '.join("'%s'" % i for i in l)
 
@@ -207,9 +173,9 @@ def _flatten(data, prefix=None):
 
     """
     if isinstance(data, dict):
-        for key, value in six.iteritems(data):
+        for key, value in data.items():
             new_key = '%s_%s' % (prefix, key) if prefix else key
-            if isinstance(value, (dict, list)):
+            if isinstance(value, (dict, list)) and value:
                 for item in _flatten(value, new_key):
                     yield item
             else:
@@ -230,7 +196,7 @@ def flatten_dict(data):
     """
     data = data.copy()
     # Try and decode any nested JSON structures.
-    for key, value in six.iteritems(data):
+    for key, value in data.items():
         if isinstance(value, six.string_types):
             try:
                 data[key] = json.loads(value)
@@ -246,7 +212,7 @@ def print_dict(d, dict_property="Property", dict_value="Value", wrap=0):
     for k, v in sorted(d.items()):
         # convert dict to str to check length
         if isinstance(v, (dict, list)):
-            v = jsonutils.dumps(v)
+            v = jsonutils.dumps(v, ensure_ascii=False)
         if wrap > 0:
             v = textwrap.fill(six.text_type(v), wrap)
         # if value has a newline, add in multiple rows
@@ -332,7 +298,7 @@ def find_resource(manager, name_or_id, wrap_exception=True, **find_args):
         raise exceptions.NotFound(404, msg)
 
 
-def _format_servers_list_networks(server):
+def format_servers_list_networks(server):
     output = []
     for (network, addresses) in server.networks.items():
         if len(addresses) == 0:
@@ -344,7 +310,7 @@ def _format_servers_list_networks(server):
     return '; '.join(output)
 
 
-def _format_security_groups(groups):
+def format_security_groups(groups):
     return ', '.join(group['name'] for group in groups)
 
 
@@ -360,7 +326,7 @@ def _format_field_name(attr):
     return ': '.join(parts)
 
 
-def _make_field_formatter(attr, filters=None):
+def make_field_formatter(attr, filters=None):
     """
     Given an object attribute, return a formatted field name and a
     formatter suitable for passing to print_list.
@@ -406,28 +372,12 @@ def do_action_on_many(action, resources, success_msg, error_msg):
             print(success_msg % resource)
         except Exception as e:
             failure_flag = True
-            print(e)
+            print(encodeutils.safe_encode(six.text_type(e)))
 
     if failure_flag:
         raise exceptions.CommandError(error_msg)
 
 
-def _load_entry_point(ep_name, name=None):
-    """Try to load the entry point ep_name that matches name."""
-    for ep in pkg_resources.iter_entry_points(ep_name, name=name):
-        try:
-            # FIXME(dhellmann): It would be better to use stevedore
-            # here, since it abstracts this difference in behavior
-            # between versions of setuptools, but this seemed like a
-            # more expedient fix.
-            if hasattr(ep, 'resolve') and hasattr(ep, 'require'):
-                return ep.resolve()
-            else:
-                return ep.load(require=False)
-        except (ImportError, pkg_resources.UnknownExtra, AttributeError):
-            continue
-
-
 def is_integer_like(val):
     """Returns validation of a value as an integer."""
     try:
@@ -452,11 +402,9 @@ def record_time(times, enabled, *args):
     """Record the time of a specific action.
 
     :param times: A list of tuples holds time data.
-    :type times: list
     :param enabled: Whether timing is enabled.
-    :type enabled: bool
-    :param *args: Other data to be stored besides time data, these args
-                  will be joined to a string.
+    :param args: Other data to be stored besides time data, these args
+        will be joined to a string.
     """
     if not enabled:
         yield
@@ -467,11 +415,17 @@ def record_time(times, enabled, *args):
         times.append((' '.join(args), start, end))
 
 
-def get_function_name(func):
-    if six.PY2:
-        if hasattr(func, "im_class"):
-            return "%s.%s" % (func.im_class, func.__name__)
-        else:
-            return "%s.%s" % (func.__module__, func.__name__)
-    else:
-        return "%s.%s" % (func.__module__, func.__qualname__)
+def prepare_query_string(params):
+    """Convert dict params to query string"""
+    # Transform the dict to a sequence of two-element tuples in fixed
+    # order, then the encoded string will be consistent in Python 2&3.
+    if not params:
+        return ''
+    params = sorted(params.items(), key=lambda x: x[0])
+    return '?%s' % parse.urlencode(params) if params else ''
+
+
+def get_url_with_filter(url, filters):
+    query_string = prepare_query_string(filters)
+    url = "%s%s" % (url, query_string)
+    return url
